/*****************************************************************

Copyright (c) 1996-2000 the kicker authors. See file AUTHORS.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************/

#include "global.h"

#include <qapplication.h>
#include <qfile.h>
#include <qpopupmenu.h>
#include <qregexp.h>

#include <kstandarddirs.h>
#include <kservice.h>

#include <kio/netaccess.h>

KPanelExtension::Position directionToPosition(KPanelApplet::Direction d )
{
    switch (d)
    {
        case KPanelApplet::Down:
            return KPanelExtension::Top;
        break;

        case KPanelApplet::Left:
            return KPanelExtension::Right;
        break;

        case KPanelApplet::Right:
            return KPanelExtension::Left;
        break;

        case KPanelApplet::Up:
        default:
            return KPanelExtension::Bottom;
        break;
    }
}

KPanelExtension::Position directionToPopupPosition(KPanelApplet::Direction d)
{
    switch (d)
    {
        case KPanelApplet::Up:
            return KPanelExtension::Top;
        break;

        case KPanelApplet::Down:
            return KPanelExtension::Bottom;
        break;

        case KPanelApplet::Left:
            return KPanelExtension::Left;
        break;

        case KPanelApplet::Right:
        default:
            return KPanelExtension::Right;
        break;
    }
}

KPanelApplet::Direction positionToDirection(KPanelExtension::Position p)
{
    switch (p)
    {
        case KPanelExtension::Top:
            return KPanelApplet::Down;
        break;

        case KPanelExtension::Right:
            return KPanelApplet::Left;
        break;

        case KPanelExtension::Left:
            return KPanelApplet::Right;
        break;

        case KPanelExtension::Bottom:
        default:
            return KPanelApplet::Up;
        break;
    }
}

int sizeValue(KPanelExtension::Size s)
{
    switch (s)
    {
        case KPanelExtension::SizeTiny:
            return 24;
        break;

        case KPanelExtension::SizeSmall:
            return 30;
        break;

        case KPanelExtension::SizeNormal:
            return 46;
        break;

        case KPanelExtension::SizeLarge:
        default:
            return 58;
        break;
    }
}

QString newDesktopFile(const KURL& url)
{
   QString base = url.fileName();
   if (base.endsWith(".desktop"))
      base.truncate(base.length()-8);
   QRegExp r("(.*)(?=-\\d+)");
   if (r.search(base) > -1)
      base = r.cap(1);

   QString file = base + ".desktop";

   for(int n = 1; ++n; )
   {
      QString path = locate("appdata", file);
      if (path.isEmpty())
         break;

      file = QString("%2-%1.desktop").arg(n).arg(base);
   }
   file = locateLocal("appdata", file);
   return file;
}

QString copyDesktopFile(const KURL& url)
{
   QString file = newDesktopFile(url);
   KURL dest;
   dest.setPath(file);
   KIO::NetAccess::upload(url.path(), dest, 0);
   return file;
}

QPoint popupPosition(KPanelApplet::Direction d, const QWidget* popup, const QWidget* source)
{
	if( !QApplication::reverseLayout() || d == KPanelApplet::Left || d == KPanelApplet::Right ) {
		return popupPosition( d, popup, source, QPoint( 0, 0 ) );
	} else {
		return popupPosition( d, popup, source, QPoint( source->width(), source->height() ) );
	}
}

QPoint popupPosition(KPanelApplet::Direction d, const QWidget* popup, const QWidget* source, const QPoint& p)
{
//	kdDebug(1210) << "popupPosition():" << endl;
//	kdDebug(1210) << m->width() << ", " << m->height() << endl;

	QRect r;
	if( source->isTopLevel() ) {
		r = source->geometry();
	} else {
		r = QRect( source->mapToGlobal( QPoint( 0, 0 ) ),
		           source->mapToGlobal( QPoint( source->width(), source->height() ) ) );
		switch( d ) {
		case KPanelApplet::Left:
		case KPanelApplet::Right:
			r.setLeft( source->topLevelWidget()->x() );
			r.setWidth( source->topLevelWidget()->width() );
			break;
		case KPanelApplet::Up:
		case KPanelApplet::Down:
			r.setTop( source->topLevelWidget()->y() );
			r.setHeight( source->topLevelWidget()->height() );
			break;
		}
	}
	switch( d ) {
	case( KPanelApplet::Left ):
		return QPoint( r.left() - popup->width(), r.top() + p.y()      );
	case( KPanelApplet::Right ):
		return QPoint( r.right() + 1,         r.top() + p.y()      );
	case( KPanelApplet::Down ):
		return QApplication::reverseLayout() ?
			QPoint( r.right() - popup->width() - (r.width() - p.x()) + 1, r.bottom() + 1 ) :
			QPoint( r.left() + p.x(),      r.bottom() + 1       );
	default:
	case( KPanelApplet::Up ):
		return QApplication::reverseLayout() ?
			QPoint( r.right() - popup->width() - (r.width() - p.x()) + 1, r.top() - popup->height() ) :
			QPoint( r.left() + p.x(),      r.top() - popup->height() );
	}
}


