/*****************************************************************

Copyright (c) 2004 Zack Rusin <zrusin@kde.org>
                   Sami Kyostil <skyostil@kempele.fi>
                   Aaron J. Seigo <aseigo@kde.org>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************/

#ifndef KICKER_TIP_H
#define KICKER_TIP_H

#include <qbitmap.h>
#include <qpixmap.h>
#include <qtimer.h>
#include <qwidget.h>

class QPaintEvent;
class QSimpleRichText;
class QTimer;

class KickerTip : public QWidget
{
    Q_OBJECT

public:
    static KickerTip* the();

    enum MaskEffect { Plain, Dissolve };
    void display(const QString& message,
                 const QString& subtext,
                 const QPixmap& icon,
                 MaskEffect mEff = Dissolve,
                 int durationMs = 4000);

    void tipFor(QWidget* w);
    bool isTippingFor(QWidget* w) const;
    void untipFor(QWidget* w);
    static void enableTipping(bool tip);
    static bool tippingEnabled();
    void hide();

protected:
    KickerTip( QWidget * parent );
    void paintEvent(QPaintEvent * e);
    void mousePressEvent(QMouseEvent * e);
    void plainMask();
    void dissolveMask();
    void displayInternal();

protected slots:
    void internalUpdate();

private:
    QBitmap m_mask;
    QPixmap m_pixmap;
    QPixmap m_icon;
    MaskEffect m_maskEffect;
    QSimpleRichText* m_richText;

    int m_dissolveSize;
    int m_dissolveDelta;

    QTimer m_timer;
    QTimer m_frameTimer;
    bool m_dirty;

    QWidget* m_tippingFor;

    static KickerTip* m_self;
    static int m_tippingEnabled;
};

#endif
